@@ -16,7 +16,7 @@ import atexit
 
 
 def start_headless(
-    mm_app_path, config_file, java_loc=None, core_log_path=None, buffer_size_mb=1024
+    mm_app_path, config_file, java_loc=None, core_log_path=None, buffer_size_mb=1024, port=Bridge.DEFAULT_PORT
 ):
     """
     Start a Java process that contains the neccessary libraries for pycro-manager to run,
@@ -42,6 +42,8 @@ def start_headless(
             Path to where core log files should be created
         buffer_size_mb : int
             Size of circular buffer in MB in MMCore
+        port : int
+            Default port to use for ZMQServer
     """
 
     classpath = '"' + mm_app_path + '/plugins/Micro-Manager/*"'
@@ -60,16 +62,18 @@ def start_headless(
             classpath,
             "-Dsun.java2d.dpiaware=false",
             "-Xmx2000m",
+
             # This is used by MM desktop app but breaks things on MacOS...Don't think its neccessary
             # "-XX:MaxDirectMemorySize=1000",
             "org.micromanager.remote.HeadlessLauncher",
+            str(port)
         ]
     )
     # make sure Java process cleans up when Python process exits
     atexit.register(lambda: p.terminate())
 
     # Initialize core
-    bridge = Bridge()
+    bridge = Bridge(port=port)
     core = bridge.get_core()
 
     core.wait_for_system()
@@ -86,7 +90,7 @@ def start_headless(
 ### These functions outside class to prevent problems with pickling when running them in differnet process
 
 
-def _event_sending_fn(event_port, event_queue, debug=False):
+def _event_sending_fn(bridge_port, event_port, event_queue, debug=False):
     """
 
     Parameters
@@ -102,7 +106,7 @@ def _event_sending_fn(event_port, event_queue, debug=False):
     -------
 
     """
-    bridge = Bridge(debug=debug)
+    bridge = Bridge(debug=debug, port=bridge_port)
     event_socket = bridge._connect_push(event_port)
     while True:
         events = event_queue.get(block=True)
@@ -118,7 +122,7 @@ def _event_sending_fn(event_port, event_queue, debug=False):
             print("sent events")
 
 
-def _acq_hook_startup_fn(pull_port, push_port, hook_connected_evt, event_queue, hook_fn, debug):
+def _acq_hook_startup_fn(bridge_port, pull_port, push_port, hook_connected_evt, event_queue, hook_fn, debug):
     """
 
     Parameters
@@ -140,7 +144,7 @@ def _acq_hook_startup_fn(pull_port, push_port, hook_connected_evt, event_queue,
     -------
 
     """
-    bridge = Bridge(debug=debug)
+    bridge = Bridge(debug=debug, port=bridge_port)
 
     push_socket = bridge._connect_push(pull_port)
     pull_socket = bridge._connect_pull(push_port)
@@ -178,7 +182,7 @@ def _acq_hook_startup_fn(pull_port, push_port, hook_connected_evt, event_queue,
 
 
 def _processor_startup_fn(
-    pull_port, push_port, sockets_connected_evt, process_fn, event_queue, debug
+    bridge_port, pull_port, push_port, sockets_connected_evt, process_fn, event_queue, debug
 ):
     """
 
@@ -201,7 +205,7 @@ def _processor_startup_fn(
     -------
 
     """
-    bridge = Bridge(debug=debug)
+    bridge = Bridge(debug=debug, port=bridge_port)
     push_socket = bridge._connect_push(pull_port)
     pull_socket = bridge._connect_pull(push_port)
     if debug:
@@ -304,6 +308,7 @@ class Acquisition(object):
         magellan_explore=False,
         process=False,
         saving_queue_size=20,
+        port=Bridge.DEFAULT_PORT,
         debug=False,
         core_log_debug=False,
     ):
@@ -374,12 +379,15 @@ class Acquisition(object):
             The number of images to queue (in memory) while waiting to write to disk. Higher values should
             in theory allow sequence acquisitions to go faster, but requires the RAM to hold images while
             they are waiting to save
+        port :
+            Allows overriding the defualt port for using Java side servers on a different port
         debug : bool
             whether to print debug messages
         core_log_debug : bool
             Print debug messages on java side in the micro-manager core log
         """
-        self.bridge = Bridge(debug=debug)
+        self.bridge = Bridge(debug=debug, port=port)
+        self._bridge_port = port
         self._debug = debug
         self._dataset = None
         self._finished = False
@@ -434,35 +442,36 @@ class Acquisition(object):
             self.disk_location = storage.get_disk_location()
 
         if image_process_fn is not None:
-            processor = self.bridge.construct_java_object(
+            java_processor = self.bridge.construct_java_object(
                 "org.micromanager.remote.RemoteImageProcessor"
             )
-            self._remote_acq.add_image_processor(processor)
-            self._start_processor(processor, image_process_fn, self._event_queue, process=process)
+            self._remote_acq.add_image_processor(java_processor)
+            self._processor_thread = self._start_processor(java_processor, image_process_fn, self._event_queue, process=process)
 
+        self._hook_threads = []
         if event_generation_hook_fn is not None:
             hook = self.bridge.construct_java_object(
                 "org.micromanager.remote.RemoteAcqHook", args=[self._remote_acq]
             )
-            self._start_hook(hook, event_generation_hook_fn, self._event_queue, process=process)
+            self._hook_threads.append(self._start_hook(hook, event_generation_hook_fn, self._event_queue, process=process))
             self._remote_acq.add_hook(hook, self._remote_acq.EVENT_GENERATION_HOOK)
         if pre_hardware_hook_fn is not None:
             hook = self.bridge.construct_java_object(
                 "org.micromanager.remote.RemoteAcqHook", args=[self._remote_acq]
             )
-            self._start_hook(hook, pre_hardware_hook_fn, self._event_queue, process=process)
+            self._hook_threads.append(self._start_hook(hook, pre_hardware_hook_fn, self._event_queue, process=process))
             self._remote_acq.add_hook(hook, self._remote_acq.BEFORE_HARDWARE_HOOK)
         if post_hardware_hook_fn is not None:
             hook = self.bridge.construct_java_object(
                 "org.micromanager.remote.RemoteAcqHook", args=[self._remote_acq]
             )
-            self._start_hook(hook, post_hardware_hook_fn, self._event_queue, process=process)
+            self._hook_threads.append(self._start_hook(hook, post_hardware_hook_fn, self._event_queue, process=process))
             self._remote_acq.add_hook(hook, self._remote_acq.AFTER_HARDWARE_HOOK)
         if post_camera_hook_fn is not None:
             hook = self.bridge.construct_java_object(
                 "org.micromanager.remote.RemoteAcqHook", args=[self._remote_acq]
             )
-            self._start_hook(hook, post_camera_hook_fn, self._event_queue, process=process)
+            self._hook_threads.append(self._start_hook(hook, post_camera_hook_fn, self._event_queue, process=process))
             self._remote_acq.add_hook(hook, self._remote_acq.AFTER_CAMERA_HOOK)
 
         self._remote_acq.start()
@@ -475,16 +484,16 @@ class Acquisition(object):
         if magellan_acq_index is None and not magellan_explore:
             self.event_port = self._remote_acq.get_event_port()
 
-            self.event_process = threading.Thread(
+            self._event_thread = threading.Thread(
                 target=_event_sending_fn,
-                args=(self.event_port, self._event_queue, self._debug),
+                args=(self._bridge_port, self.event_port, self._event_queue, self._debug),
                 name="Event sending",
             )
-            self.event_process.start()
+            self._event_thread.start()
 
         if storage_monitor_callback_fn is not None:
             self._dataset = Dataset(remote_storage_monitor=self._remote_acq.get_storage_monitor())
-            self._dataset._add_storage_monitor_fn(
+            self._storage_monitor_thread =  self._dataset._add_storage_monitor_fn(
                 callback_fn=storage_monitor_callback_fn, debug=self._debug
             )
 
@@ -511,7 +520,7 @@ class Acquisition(object):
             # Load remote storage
             self._dataset = Dataset(remote_storage_monitor=self._remote_acq.get_storage_monitor())
             # Monitor image arrival so they can be loaded on python side, but with no callback function
-            self._dataset._add_storage_monitor_fn(callback_fn=None, debug=self._debug)
+            self._storage_monitor_thread = self._dataset._add_storage_monitor_fn(callback_fn=None, debug=self._debug)
 
         return self._dataset
 
@@ -520,6 +529,21 @@ class Acquisition(object):
         while not self._remote_acq.is_finished():
             time.sleep(0.1)
         self._remote_acq = None
+
+        # Wait on all the other threads to shut down properly
+        if hasattr(self, '_storage_monitor_thread'):
+            self._storage_monitor_thread.join()
+
+        for hook_thread in self._hook_threads:
+            hook_thread.join()
+
+        if hasattr(self, '_event_thread'):
+            self._event_thread.join()
+
+        if hasattr(self, '_storage_monitor_thread'):
+            self._storage_monitor_thread.join()
+
+
         self._finished = True
 
     def acquire(self, events, keep_shutter_open=False):
@@ -580,6 +604,7 @@ class Acquisition(object):
             target=_acq_hook_startup_fn,
             name="AcquisitionHook",
             args=(
+                self._bridge_port,
                 pull_port,
                 push_port,
                 hook_connected_evt,
@@ -592,6 +617,7 @@ class Acquisition(object):
         hook_thread.start()
 
         hook_connected_evt.wait()  # wait for push/pull sockets to connect
+        return hook_thread
 
     def _start_processor(self, processor, process_fn, event_queue, process):
         """
@@ -619,9 +645,10 @@ class Acquisition(object):
         pull_port = processor.get_pull_port()
         push_port = processor.get_push_port()
 
-        self.processor_thread = (multiprocessing.Process if process else threading.Thread)(
+        processor_thread = (multiprocessing.Process if process else threading.Thread)(
             target=_processor_startup_fn,
             args=(
+                self._bridge_port,
                 pull_port,
                 push_port,
                 sockets_connected_evt,
@@ -631,10 +658,11 @@ class Acquisition(object):
             ),
             name="ImageProcessor",
         )
-        self.processor_thread.start()
+        processor_thread.start()
 
         sockets_connected_evt.wait()  # wait for push/pull sockets to connect
         processor.start_push()
+        return processor_thread
 
 
 def multi_d_acquisition_events(
